;xcf,png,gifの画像形式とディレクトリを指定し、ディレクトリ内の画像を一括でしょりする。
;処理内容 余白削除、余白を上下左右10pxに設定、画像サイズが奇数にならないように調整、pngで保存
;LINEスタンプ作成において、画像の余白各10px、画像サイズが偶数と言う仕様のため、一括で処理するスクリプトを作成した。

;選択範囲のサイズを取得してトリミングする;{{{
(define (trim-selection image)
  (let* (
    (x1 (cadr (gimp-selection-bounds image)));2番めの返り値X1を取り出す
    (y1 (caddr (gimp-selection-bounds image)));3番めの返り値y1を取り出す
    (x2 (car (cdddr (gimp-selection-bounds image))));4番めの返り値x2を取り出す
    (y2 (cadr (cdddr (gimp-selection-bounds image))));5番めの返り値y2を取り出す
    (width (- x2 x1));選択範囲の幅
    (height (- y2 y1));選択範囲の高さ
    (x1 (- x1))
    (y1 (- y1))
    )
    (gimp-image-resize image width height x1 y1 );トリミング
  )
 );}}}

 ;余白を上下左右に10pxつける;{{{
(define (yohaku image)
  (let* (
    (width (car (gimp-image-width image)));現在の画像サイズ(幅)を取得
    (height (car (gimp-image-height image)));現在の画像サイズ(高さ)を取得
    (width (+ width 20))
    (height (+ height 20))
    )
  (gimp-image-resize image width height 10 10 )
  )
);}}}

;画像サイズが奇数だった場合、偶数に修正する;{{{
(define (kisu-to-gusu image)
  (let*  (
    (width (car (gimp-image-width image)));現在の画像サイズ(幅)を取得
    (height (car (gimp-image-height image)));現在の画像サイズ(高さ)を取得
    )
    (if 
      (odd? width);画像幅が奇数か確認
      (set! width (+ width 1));奇数ならwidthを+1する
    )
    (if 
      (odd? height);画像高さが奇数か確認
      (set! height (+ height 1));奇数ならheightを+1する
    )
    (gimp-image-resize image width height 0 0 );リサイズ実行
  )
);}}}

;pngファイルに保存する処理;{{{
(define (png-save image layer filename)
  (let* (
    (length (string-length filename));ファイル名の長さの変数を取得
    (newFileName (string-append (substring filename 0 (- length 4)) ".png"));ファイル名の末尾4文字を削除(拡張子".xcf"として、拡張子を削除し、".png"追加)
    )
    (gimp-layer-resize-to-image-size layer);レイヤーサイズをイメージサイズに合わせる。これがないと、リサイズした画像を保存してもサイズが変わっていなかった。
    (file-png-save2 RUN-NONINTERACTIVE image layer newFileName newFileName 0 9 1 0 0 1 1 0 1);filenameで保存。元ファイルと同じフォルダに同じ名前で保存される。引数の6～14番目はgimpのpngをエクスポートするときのダイアログのチェックボックスに対応している。)
       ;RUN-NONINTERACTIVEにしているとなぜかアクティブレイヤーしか保存されない。
  )
);}}}

;画像サイズが幅370,高さ350より大きい場合トリミングする処理
(define (trim-image image)
  (let* (
    (width (car (gimp-image-width image)));現在の画像サイズ(幅)を取得
    (height (car (gimp-image-height image)));現在の画像サイズ(高さ)を取得
    (offset-w 0)
    (offset-h 0)
    )
    (if (> width 370)
      (begin
        (set! offset-w (/ (- width 370) -2))
        (set! width 370)
      )
    )
    (if (> height 320)
      (begin
        (set! offset-h (/ (- height 320) -2))
        (set! height 320)
      )
    )
    (gimp-image-resize image width height offset-w offset-h )
  )
)


(define (script-fu-line-stamp-resize-png-save-kk path-0 inDir-0)
  (let* (;{{{    
    (varLoadStr "");ファイルの拡張子を入れる変数
    (varFileList 0);ファイルリスト(ファイルの一覧のファイル名を入れる変数)
    (inDir "");画像を読み込むパスを入れる変数
    (filename "")
    (short_filename "")
    (image "")
    (layer "")
    (buffer-name "")
    )
    (if (> (string-length path-0) 1);テキスト入力ボックスに文字が入力されていれば、inDirに文字列をセット、入力されていなければ選択したフォルダをinDirにセット。(テキスト入力ボックスが優先される)
      (set! inDir path-0)
      (set! inDir inDir-0)
    )
    (set! varFileList (cadr (file-glob (string-append inDir "\\*" varLoadStr)  1)));ファイルリスト

    (while (not (null? varFileList));{{{
      (set! filename (car varFileList));ファイルリストの1番目をfilenameに入れる
      (set! short_filename (substring filename (+ (string-length inDir) 1) (string-length filename)));ファイル名(フルパス)からパスを除去
      (set! image (car (gimp-file-load RUN-NONINTERACTIVE filename filename)));ロードしたイメージをimageに入れる
      (set! layer (car (gimp-image-get-active-layer image)))
      
      (gimp-selection-none image);選択範囲をクリア
      (set! buffer-name (car (gimp-edit-named-copy-visible image "snapshot")));現在の可視イヤーをコピー
      (set! image (car (gimp-edit-named-paste-as-new buffer-name)))
      (set! layer (car (gimp-image-get-active-layer image)))
        
      (gimp-selection-layer-alpha layer);レイヤーの透明部分を選択
      (trim-selection image);選択範囲のサイズを取得してトリミング
      (yohaku image);余白を上下左右に10pxつける
      (kisu-to-gusu image);画像サイズが奇数だった場合、偶数に修正する
      (trim-image image);画像サイズが幅370,高さ350より大きい場合トリミングする処理
      (png-save image layer filename);pngファイルに保存する処理

      (gimp-image-delete image)
      (gimp-buffer-delete buffer-name)
      (set! varFileList (cdr varFileList));ファイルリストの2番めを1番目に入れ替える
    );while}}}
    (gimp-patterns-refresh)
             
  );}}}
)

    ;------------------------------------------------------------------

    (script-fu-register "script-fu-line-stamp-resize-png-save-kk" ;func name,関数の名前;{{{
                        "KK LINE stamp resize png save" ;menu label
                        "" ;description,説明
                        "Kawahara" ;author,作者
                        "" ;copyright notice,著作権
                        "2016-08-10" ;date created,作成日
                        "" ;image type that the script works on,画像の種類
                        ;    SF-IMAGE "Image" 0
                        ;    SF-DRAWABLE "Drawable" 0
                        ;SF-STRING     "フォルダのパスを入力、又は下記よりフォルダを選択してください。"
                        SF-STRING     "フォルダのパスを入力(末尾に\"\\\"は付けないこと)" ""
                        SF-DIRNAME    "フォルダを選択(上のボックスに入力した場合、ボックスが優先されます)" ""
                        )
    (script-fu-menu-register "script-fu-line-stamp-resize-png-save-kk"
                             "<Image>/Script-Fu");}}}

